import { createElementSize } from "@solid-primitives/resize-observer";
import {
  JSX,
  Show,
  createRenderEffect,
  createSignal,
  splitProps,
  type Component,
  type ParentProps,
} from "solid-js";
import "./Scaffold.css";

type ScaffoldProps = ParentProps<
  {
    topbar?: JSX.Element;
    fab?: JSX.Element;
    bottom?: JSX.Element;
  } & JSX.HTMLElementTags["div"]
>;

/**
 * The passthrough props are passed to the content container.
 */
const Scaffold: Component<ScaffoldProps> = (props) => {
  const [managed, rest] = splitProps(props, [
    "topbar",
    "fab",
    "bottom",
    "children",
    "ref",
    "class",
  ]);
  const [topbarElement, setTopbarElement] = createSignal<HTMLElement>();

  const topbarSize = createElementSize(topbarElement);

  return (
    <div
      class={`Scaffold ${managed.class || ""}`}
      ref={(e) => {
        createRenderEffect(() => {
          e.style.setProperty(
            "--scaffold-topbar-height",
            (topbarSize.height?.toString() ?? 0) + "px",
          );
        });

        if (managed.ref) {
          (managed.ref as (val: typeof e) => void)(e);
        }
      }}
      {...rest}
    >
      <Show when={props.topbar}>
        <div class="topbar" ref={setTopbarElement} role="presentation">
          {props.topbar}
        </div>
      </Show>
      <Show when={props.fab}>
        <div class="fab-dock" role="presentation">
          {props.fab}
        </div>
      </Show>

      {managed.children}

      <Show when={props.bottom}>
        <div class="bottom-dock" role="presentation">
          {props.bottom}
        </div>
      </Show>
    </div>
  );
};

export default Scaffold;
